#include "hardware_t.h"
#include "tc_vars.h"
#include "flash_t.h"
#include "display_t.h"
#include "PC-comms.h"

/*********************************************************************
Local functions
*********************************************************************/
int handle_extended_command(unsigned char *);

//##########################################################################################################################################################
// BF new screenshot function for black/white screenshots written by Niklas
// BF optimized by Falk
void	rle_enc(unsigned char c, int	init)
{
	static	unsigned short		llength;
	static	short	lastbyte;
	static unsigned short	maxlen=0xFEFE;
	unsigned	char	cout;

	if (init) {
		llength=0; lastbyte=-1;
		return; }
	if (c==lastbyte) {
		llength++;
		if (llength>maxlen) {
			putchar(0xFF);						//Escape
			putchar(llength>>8);  	//MSB
			putchar(llength); 	 		//LSB
			llength=0;
			lastbyte=-1;
			}
		}
	else {												//c != lastbyte
		if (llength > 0) {								//if llength > 0, write esc, LSB, MSB
			putchar(0xFF); 		 		//ESC
			putchar(llength>>8);		//MSB
			putchar(llength);			//LSB
			llength=0;
		}
		if (c==0xFF) { putchar(c);  }	//Escape
		putchar(c); 
		lastbyte=c;
		}
}
//##########################################################################################################################################################
// BF new screenshot function for black/white screenshots written by Niklas
// nevm
// BF optimized by Falk
void PCComms::SCREENSHOT_BW(void)
{
	int x, y;
	int	bitp=0;
	int bit, pos, offset;
	unsigned long *planes[] = {
		Grid_Plane,
		UI_Plane2, UI_Plane5, // UI_Plane1, UI_Plane3, UI_Plane4
		Channel_Plane1, Channel_Plane2, Channel_Plane3, Channel_Plane4,
		Channel_Math_Plane,
		Memory_Plane1, Memory_Plane2, Memory_Plane3,
		Marker_Plane1, Marker_Plane2,
		NULL
	};
	unsigned long *plane;
	int i, set, out=0;

	rle_enc(0,1);				//Initialize rle
	(void)putchar('\n');
	(void)putchar('S');			//Send start-of-transfer marker
	(void)putchar(0xFF);			//Send start-of-transfer marker
	(void)putchar(1);			//Send dumptye 1 monochrome


	for (y = 0; y < 480; y++) {
		for (x = 0; x < 640; x++) {
			offset = x >> 5;		// x div 32
			pos = Display_Line_Adresses[y] + offset;
			bit = x - (offset << 5);	// x - (offset mod 32)

			set = 0;
//Walk through planes but stop as soon as a pixel is set.
			for (i = 0; (plane = planes[i]) != NULL; i++) {
				if (*(plane + pos) & BitMasks2[bit]) {
					set=1;
					break;
				}
			}

/*********************************************************************
* Changed (dl3daz):
* shift bits into one byte for rle-encoding
*********************************************************************/
			if (set) out |= (1 << bitp);
			bitp++;
			if (bitp==8) {
			rle_enc(out,0);
				out=0;
				bitp=0; }
		}
	}
	rle_enc(0xFF,0);			//Send End-of-plane marker
	rle_enc('E',0);				//Send End-of-transfer-marker
	rle_enc('\n',0);

}

//##########################################################################################################################################################
// BF new screenshot function for colored screenshots written by Niklas
// nevm
// BF optimized by Falk
void PCComms::SCREENSHOT(void)
{
	short x, y;
	int bit, pos, offset;
	int	lastbyte,length,bitp=0;			//FW Last Byte, Number of same bytes
	unsigned long *planes[] = {
		Grid_Plane,
		UI_Plane1, UI_Plane2, UI_Plane3, UI_Plane4, UI_Plane5,
		Channel_Plane1, Channel_Plane2, Channel_Plane3, Channel_Plane4,
		Channel_Math_Plane,
		Memory_Plane1, Memory_Plane2, Memory_Plane3,	//BF -> not needed!!!
		Marker_Plane1, Marker_Plane2,
		NULL
	};
	unsigned long *plane;
	int i, set, out=0;

	rle_enc(0,1);					//Initialize rle
	(void)putchar('\n');
	(void)putchar('S');				//Send start-of-transfer marker
	(void)putchar(0xFF);				//Send start-of-transfer marker
	(void)putchar(0);				//Send dumptye 0 Color

	for (i = 0; (plane = planes[i]) != NULL; i++) {
		for (y = 0; y < 480; y++) {
			for (x = 0; x < 640; x++) {
				offset = x >> 5;		// x div 32
				pos = Display_Line_Adresses[y] + offset;
				bit = x - (offset << 5);	// x - (offset mod 32)
/*********************************************************************
* Changed (dl3daz):
* shift bits into one byte for rle-encoding
*********************************************************************/
				out |= (((*(plane + pos) & BitMasks2[bit]) ? 1 : 0) << bitp);
				bitp++;
				if (bitp==8) {
					rle_enc(out,0);
					out=0;
					bitp=0; }
			}
		}
		rle_enc(0xFF,0);	//Send End-of-plane marker
	}

	rle_enc('E',0);			//Send End-of-transfer-marker
	rle_enc('\n',0);
}

//##########################################################################################################################################################
// BF new .csv function for transmission of raw data written by Falk
void PCComms::DUMPCSV(void) {
	int	idx;
	rle_enc(0,1);
	(void)putchar('\n');
	(void)putchar('S');				//Send start-of-transfer marker
	(void)putchar(0xFF);				//Send start-of-transfer marker
	(void)putchar(2);				//Send dumptye 2 (CSV)
	for (idx = 0; idx < 16384; idx++) {
		putchar(SIGNAL1[idx]);
		putchar(SIGNAL2[idx]);
		putchar(SIGNAL3[idx]);
		putchar(SIGNAL4[idx]); }
	rle_enc('E',0);					//Send End-of-transfer-marker
	rle_enc('\n',0);
}


int	PCComms::handleInChar(unsigned char c) {
	static int debug=1;
	int lMenuKey = -1;

char	charTOlMenuKey[]={
//  0x0  0x1  0x2  0x3  0x4  0x5  0x6  0x7  0x8  0x9  0xA  0xB  0xC  0xD  0xE  0xF
	  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, -11, -12, -10,  -1,  -1,  -1,  -1,  -1,
//  0x10 0x11 0x12 0x13 0x14 0x15 0x16 0x17 0x18 0x19 0x1A 0x1B 0x1C 0x1D 0x1E 0x1F
	  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, -11, -12, -10,  -1,  -1,  -1,  -1,  -1,
//  ' '  '!'  '"'  '#'  '$'  '%'  '&'  '''  '('  ')'  '*'  '+'  ','  '-'  '.'  '/'
	  -2,  80,  81,  -1,  83,  84,  85,  -1,  87,  88,  -1,  -1,  69,  71,  70,  86,
//  '0'  '1'  '2'  '3'  '4'  '5'  '6'  '7'  '8'  '9'  ':'  ';'  '<'  '='  '>'  '?'
	  39,  30,  31,  32,  33,  34,  35,  36,  37,  38,  68,  67,-127,  89,-127,-127,
//  '@'  'A'  'B'  'C'  'D'  'E'  'F'  'G'  'H'  'I'  'J'  'K'  'L'  'M'  'N'  'O'
	 -1,  50,  64,  62,  52,  42,  53,  54,  55,  47,  56,  57,  58,  66,  65,  48,
//  'P'  'Q'  'R'  'S'  'T'  'U'  'V'  'W'  'X'  'Y'  'Z'  '['  '\'  ']'  '^'  '_'
	 49,  40,  43,  51,  44,  46,  63,  41,  61,  60,  45,-127,-127,-127,-127,-127,
//  '`'  'a'  'b'  'c'  'd'  'e'  'f'  'g'  'h'  'i'  'j'  'k'  'l'  'm'  'n'  'o'
	 -1,   0,  10,  16,   5,  14,   7,  23,  55,   1,   2,   3,   4,  13,  22,  26,
//  'p'  'q'  'r'  's'  't'  'u'  'v'  'w'  'x'  'y'  'z'  '{'  '|'  '}'  '~'  DEL
	 15,  19,   9,   8,  12,   6,  21,  20,  27,  17,  29,  -1,  -1,  -1,  -1,  -1};

	if (UART_NewData) {
		if (debug) printf("handleInChar(%.2X)\n\r",c);
		UART_NewData = 0;			   //reset UART ISR flag
		if (UART_RXData!=STX) {
			if (UART_RXData>0) lMenuKey=charTOlMenuKey[UART_RXData];

			if(lMenuKey != -1)
			{
				ButtonChanged = 1;

				Hardware::Buttonhandler(lMenuKey, 0);
				//return;
			}
		}
	}
	if (UartRxGuiCmd) {
		if (debug) printf("handleInChar(%.2X), UartRxGuiCmd: %d\n\r",c,UartRxGuiCmd);
		switch(UartRxGuiCmd) {		//handle 2Byte command for screenshot-App.
				case	1:
					PCComms::SCREENSHOT();
				break;
				case	2:
					PCComms::SCREENSHOT_BW();
				break;
				case	3:
					PCComms::DUMPCSV();
				break;
				case	'E':
					handle_extended_command(UartRxGuiBuf);
				break;
		}
		UartRxGuiCmd=0;
	}
}

/*********************************************************************
Handle characters received after STX-0xFF:
Returns 0 if more characters expected,
returns 1 if command is complete

All commands from GUI start with STX and 0xFF
Next Byte is a command, following three bytes a parameter.
Example: Set Timebase:
STX 0xFF	Header
'T'			Command Timebase
NN NN NN	Parameter for timebase: 1: 2ns, 2: 5ns, 3: 10ns ...
*********************************************************************/
int handle_extended_command(unsigned char *buffer) {
	unsigned char debug=1;

	if (debug) printf("extendend Command complete\n\r");
/*********************************************************************
Buffer is full. Read buffer and do what has to be done
*********************************************************************/
	switch(buffer[0]) {				//This is the command
		case 'T':					//Set/Get Timebase
			if ((buffer[1] <= MAXTIMEBASEVAL)) {

				if (debug) printf("Setting TB to %d\n\r",(int)buffer[1]);

				Selected_Timebase=buffer[1];
/*********************************************************************
				timebase_reg = tb_value[Selected_Timebase];
				Display_Timebase = Selected_Timebase + SIGNALFaktor_idx;

				if (Selected_Timebase > 7) { adc_change12_reg |= 0x01000000; }
				else { adc_change12_reg &= 0xFEFFFFFF; }

				Display::StatusUpdate();

				TimebaseChanged = 1; 
*********************************************************************/
			Hardware::ON_Timebase();
		}
		else { if (debug) printf("Invalid TB-Setting: %d\n\r",(int)buffer[1]); }
		break;	//case 'T':
	}
	return(1);	//Finished
}
