#include "hardware_t.h"
#include "tc_vars.h"
#include "flash_t.h"
#include "display_t.h"
//#include "math.h"
//###########################################################################################################################################

void Hardware::DoEnableUARTInterrupt(void)						// Enable service routine
{
	nr_installuserisr(na_uart1_irq, ISR_UART, (int)puart);				// Install ISR for UART

	puart->np_uartcontrol = np_uartcontrol_irrdy_mask;				// Enable Control Bits
#ifdef _Debug_IRQ_	
	if (Debug_Mode) printf("\nUART interrupt enabled.\n"); 				// print on console
#endif	
}
	
void Hardware::DoDisableUARTInterrupt(void)						// Disable service routine
{
	puart->np_uartcontrol = 0;							// Disable Control Bits
	nr_installuserisr(na_uart1_irq, 0, 0); 						// Install empty routine for uart irq
}
//###########################################################################################################################################
//BF cleaned ISR from useless overhead and transferred buttonhandler to keyboard interface
/*********************************************************************
/ Added state machine for GUI-commands.
/ On reception of STX (0x02) eight Bytes are read into a buffer.
/ When buffer is full, UartRxGuiCmd is set to 1
*********************************************************************/

void Hardware::ISR_UART(int context)							// UART Interrupt subroutine
{
	char rxchar;
	int status;
	static unsigned char state=0,rxbufcnt=0;

	status = puart->np_uartstatus;	
	rxchar = puart->np_uartrxdata;	
	
	puart->np_uartstatus = 0;
    	
	
	if (status & np_uartstatus_rrdy_mask)
	{
		switch (state) {
			case 0:
				if (rxchar != STX) {		//Standard handling
					UART_RXData  = rxchar;
					UART_NewData = 1;
					UI_request   = 1;
				}
				else state=1;
			break;
			case 1:
				if (rxchar == 'E') {		//Extended CMD from GUI
					state=2; }
				else {						//This is a screenshot-request
					UartRxGuiCmd=rxchar;	//This setting is caught in PC-comms.cpp
					state=0; }
			break;
			case 2:
					UartRxGuiBuf[rxbufcnt++]=rxchar;	//Store to buffer
					if (rxbufcnt==UARTRXGUISIZE) {		//If buffer is full
						UartRxGuiCmd='E';				//Mark UartRxGuiCmd
						rxbufcnt=0;						//Reset counter
						state=0; }						//and state
			break;
		}
	}
}

